# ======================================================================
#         Import modules
# ======================================================================
import os
import numpy as np
import argparse
from mpi4py import MPI
from baseclasses import AeroProblem
from adflow import ADFLOW

comm = MPI.COMM_WORLD

# Use Python's built-in Argument parser to get commandline options
parser = argparse.ArgumentParser()
parser.add_argument("--output", type=str, default="./output")
parser.add_argument("--gridFile", type=str, default="n0012.cgns")
args = parser.parse_args()

# ======================================================================
#         Specify parameters for optimization
# ======================================================================
# angle of attack
alpha = 1.5
# mach number
mach = 0.75
# cruising altitude
alt = 10000

if not os.path.exists(args.output):
    if comm.rank == 0:
        os.mkdir(args.output)

# ======================================================================
#         ADflow Set-up
# ======================================================================
# rst adflow (beg)
aeroOptions = {
    # Common Parameters
    "gridFile": args.gridFile,
    "outputDirectory": args.output,
    # Physics Parameters
    "equationType": "RANS",
    "smoother": "DADI",
    "MGCycle": "sg",
    # "nCycles": 20000,
    "nCycles": 1,
    "monitorvariables": ["resrho", "cl", "cd"],
    "useNKSolver": True,
    "useanksolver": True,
    "anksecondordswitchtol": 1e-3,
    "ankcoupledswitchtol": 1e-4,
    "nsubiterturb": 10,
    "liftIndex": 2,
    "infchangecorrection": True,
    # Convergence Parameters
    "L2Convergence": 1e-15,
    "L2ConvergenceCoarse": 1e-4,
    # Adjoint Parameters
    "adjointSolver": "GMRES",
    "adjointL2Convergence": 1e-6,
    "ADPC": True,
    "adjointMaxIter": 5000,
    "adjointSubspaceSize": 400,
    "ILUFill": 3,
    "ASMOverlap": 3,
    "outerPreconIts": 3,
    "NKSubSpaceSize": 400,
    "NKASMOverlap": 4,
    "NKPCILUFill": 4,
    "NKJacobianLag": 5,
    "nkswitchtol": 1e-6,
    "nkouterpreconits": 3,
    "NKInnerPreConIts": 3,
    "writeSurfaceSolution": False,
    "writeVolumeSolution": False,
    "frozenTurbulence": False,
    "restartADjoint": False,
    "globalpreconditioner": "multigrid"
}

# Create solver
CFDSolver = ADFLOW(options=aeroOptions, comm=comm)


# add the active slice
normal = np.array([0.0, 0.0, 1.0])
point = np.array([0.0, 0.0, 0.1])
CFDSolver.addActiveSlice(normal, point)

# Add two regular slices
span = 1.0
pos = np.array([0.2]) * span
CFDSolver.addSlices("z", pos, sliceType="relative")
# TODO when added like this, the slice indices get mixed up but this was a problem with the original code as well
pos = np.array([0.3]) * span
CFDSolver.addSlices("z", pos, sliceType="absolute")

# add another active slice to test the indices
normal = np.array([0.0, 0.0, 1.0])
point = np.array([0.0, 0.0, 0.4])
CFDSolver.addActiveSlice(normal, point)

# ======================================================================
#         Set up flow conditions with AeroProblem
# ======================================================================
ap = AeroProblem(name="fc", alpha=alpha, mach=mach, altitude=alt, areaRef=1.0, chordRef=1.0, evalFuncs=["cl", "cd"])
# Add angle of attack variable
ap.addDV("alpha", value=alpha, lower=0, upper=10.0, scale=1.0)


CFDSolver(ap)
sliceData = CFDSolver.getActiveSliceData()
print(sliceData)
